package com.tzld.piaoquan.wecom.service.Impl;

import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.wecom.common.constant.MessageConstant;
import com.tzld.piaoquan.wecom.common.exception.CustomizeException;
import com.tzld.piaoquan.wecom.model.vo.MessageTextVo;
import com.tzld.piaoquan.wecom.service.AccessTokenService;
import com.tzld.piaoquan.wecom.service.MessageService;
import com.tzld.piaoquan.wecom.utils.HttpClientUtil;
import com.tzld.piaoquan.wecom.utils.HttpPoolClient;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

import static com.tzld.piaoquan.wecom.common.constant.RedisConstant.PUSH_MESSAGE_TEXT;
import static com.tzld.piaoquan.wecom.common.constant.WeComConstant.POST_WE_COM_ADD_MSG_TEMPLATE;
import static com.tzld.piaoquan.wecom.common.enums.ExceptionCodeEnum.PARAMS_ERROR;
import static com.tzld.piaoquan.wecom.common.constant.WeComConstant.POST_MESSAGE_PUSH_URL;


@Slf4j
@Service
public class MessageServiceImpl implements MessageService {

    private static final HttpPoolClient httpPoolClientDefault = HttpClientUtil.create(30000, 30000, 2000, 5000, 5, 30000);

    @Autowired
    private AccessTokenService accessTokenService;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    public boolean pushMessage(JSONObject jsonObject) {
        try {
            String accessToken = accessTokenService.getAccessToken();
            String url = POST_MESSAGE_PUSH_URL
                    + "?access_token=" + accessToken;
            String s = httpPoolClientDefault.post(url, jsonObject.toJSONString());
            JSONObject res = JSONObject.parseObject(s);
            log.info("pushMessage res={}", res);
            Integer code = res.getInteger("errcode");
            return code == 0;
        } catch (IOException e) {
            log.error("pushMessage error", e);
        }
        return false;
    }

    @Override
    public boolean pushWeComMessage(JSONObject jsonObject) {
        try {
            String accessToken = accessTokenService.getWeComAccessToken();
            String url = POST_WE_COM_ADD_MSG_TEMPLATE
                    + "?access_token=" + accessToken;
            String s = httpPoolClientDefault.post(url, jsonObject.toJSONString());
            JSONObject res = JSONObject.parseObject(s);
            log.info("pushWeComMessage res={}", res);
            Integer code = res.getInteger("errcode");
            return code == 0;
        } catch (IOException e) {
            log.error("pushWeComMessage error", e);
        }
        return false;
    }

    @Override
    public void createMessageText(MessageTextVo messageTextVo) {
        if (messageTextVo == null || messageTextVo.getHour() == null) {
            throw new CustomizeException(PARAMS_ERROR);
        }
        if (messageTextVo.getHour() == -1) {
            redisTemplate.opsForValue().set(PUSH_MESSAGE_TEXT, messageTextVo.getText());
        } else {
            redisTemplate.opsForValue().set(PUSH_MESSAGE_TEXT, messageTextVo.getText(), messageTextVo.getHour(), TimeUnit.HOURS);
        }
    }


    @Override
    public String getMessageText() {
        String text = (String) redisTemplate.opsForValue().get(PUSH_MESSAGE_TEXT);
        if (StringUtils.isEmpty(text)) {
            return MessageConstant.guaranteedText;
        }
        return text;
    }
}
